from ._stream import (                  # NOQA; isort:skip
    ChaCha20, RC4
)
from ._block_AES import (               # NOQA; isort:skip
    AES, AES_ECB, AES_CBC, AES_CFB, AES_OFB, AES_CTR
)
from ._block_TripleDES import (         # NOQA; isort:skip
    TripleDES, TripleDES_ECB, TripleDES_CBC, TripleDES_CFB, TripleDES_OFB, TripleDES_CTR
)

__all__ = [
    'ARC4', 'ChaCha20', 'RC4',
    'AES', 'AES_ECB', 'AES_CBC', 'AES_CFB', 'AES_OFB', 'AES_CTR',
    'DES', 'DES_ECB', 'DES_CBC', 'DES_CFB', 'DES_OFB', 'DES_CTR',
    'TripleDES', 'TripleDES_ECB', 'TripleDES_CBC', 'TripleDES_CFB', 'TripleDES_OFB', 'TripleDES_CTR'
]

ARC4 = RC4                              # alias

# The Cryptography library does not provide DES, but we can use TripleDES with 64-bit key instead.
DES = TripleDES
DES_ECB = TripleDES_ECB
DES_CBC = TripleDES_CBC
DES_CFB = TripleDES_CFB
DES_OFB = TripleDES_OFB
DES_CTR = TripleDES_CTR



from . import _base, _lib


class ChaCha20(_base.StreamCipher):
    _algorithm = _lib.ChaCha20
    name = 'ChaCha20'
    key_sizes = frozenset([32])

    def __init__(self, key, *, nonce):
        self._cipher = _lib.Cipher(self._algorithm(key, nonce), None, _lib.backend)


class RC4(_base.StreamCipher):
    _algorithm = _lib.ARC4
    name = 'RC4'
    key_sizes = frozenset([5, 7, 8, 10, 16, 20, 24, 32])



from cryptography.hazmat.backends import default_backend                # NOQA; isort:skip
from cryptography.hazmat.primitives.ciphers import Cipher               # NOQA; isort:skip
from cryptography.hazmat.primitives.ciphers.algorithms import (         # NOQA; isort:skip
    ARC4, ChaCha20,
    AES, Blowfish, Camellia, CAST5, IDEA, SEED, TripleDES
)
from cryptography.hazmat.primitives.ciphers.modes import (              # NOQA; isort:skip
    ECB, CBC, CFB, OFB, CTR,
    Mode, ModeWithInitializationVector, ModeWithNonce
)

backend = default_backend()



import abc
import inspect
import os

__all__ = ['PKCS7', 'ANSIX923', 'ISO10126']


class PadderContext(metaclass=abc.ABCMeta):
    """Abstract base class for padder context."""

    def __init__(self, block_size):
        """Initialize the current context."""

        self.block_size = block_size
        self._size = 0

    def update(self, data):
        """Update the current context."""

        self._size += len(data)
        return data

    def finalize(self):
        """Finalize the current context and return the rest of the data."""

        padded_size = self.block_size - (self._size % self.block_size)
        return self._pad(padded_size)

    @staticmethod
    @abc.abstractmethod
    def _pad(padded_size):
        """Return the padding."""


class UnpadderContext(metaclass=abc.ABCMeta):
    """Abstract base class for unpadder context."""

    def __init__(self, block_size):
        """Initialize the current context."""

        self.block_size = block_size
        self._buf = b''

    def update(self, data):
        """Update the current context."""

        self._buf += data
        remaining_size = (len(self._buf) % self.block_size) or self.block_size
        result = self._buf[:-remaining_size]
        self._buf = self._buf[-remaining_size:]
        return result

    def finalize(self):
        """Finalize the current context and return the rest of the data."""

        if len(self._buf) != self.block_size:
            raise ValueError('invalid padding')
        padded_size = self._buf[-1]
        if padded_size > self.block_size:
            raise ValueError('invalid padding')
        self._check(self._buf, padded_size)
        return self._buf[:-padded_size]

    @staticmethod
    @abc.abstractmethod
    def _check(buffer, padded_size):
        """Check the padding."""


class Padding(metaclass=abc.ABCMeta):
    """Abstract base class for padding."""

    @property
    @abc.abstractmethod
    def _padder_cls(self):
        """The class of padder context."""

    @property
    @abc.abstractmethod
    def _unpadder_cls(self):
        """The class of unpadder context."""

    def __init__(self, block_size):
        """Prepare the padding context."""

        if not isinstance(block_size, int):
            raise TypeError('block_size must be int, got {}'.format(type(block_size).__name__))
        if block_size < 1 or block_size > 255:
            raise ValueError('block_size must be in [1, 255], got {}'.format(block_size))

        self.block_size = block_size

    def padder(self):
        """Return the padder context."""

        return self._padder_cls(self.block_size)

    def unpadder(self):
        """Return the unpadder context."""

        return self._unpadder_cls(self.block_size)

    class _PaddingWrapper(object):
        def __init__(self, padded_ctx, padding_ctx):
            self.padded_ctx = padded_ctx
            self.padding_ctx = padding_ctx

        def update(self, data):
            return self.padded_ctx.update(self.padding_ctx.update(data))

        def finalize(self):
            temp = self.padded_ctx.update(self.padding_ctx.finalize())
            return temp + self.padded_ctx.finalize()

    def wrap_encryptor(self, encryptor_ctx):
        return self._PaddingWrapper(encryptor_ctx, self.padder())

    def wrap_decryptor(self, decryptor_ctx):
        return self._PaddingWrapper(self.unpadder(), decryptor_ctx)


# ==============
# PKCS#7 Padding
# ==============


class _PKCS7Padder(PadderContext):
    @staticmethod
    def _pad(padded_size):
        return padded_size.to_bytes(1, 'big') * padded_size


class _PKCS7Unpadder(UnpadderContext):
    @staticmethod
    def _check(buffer, padded_size):
        for i in range(2, padded_size + 1):
            if buffer[-i] != padded_size:
                raise ValueError('invalid padding')


class PKCS7(Padding):
    _padder_cls = _PKCS7Padder
    _unpadder_cls = _PKCS7Unpadder


# ==================
# ANSI X9.23 Padding
# ==================


class _ANSIX923Padder(PadderContext):
    @staticmethod
    def _pad(padded_size):
        return b'\x00' * (padded_size - 1) + padded_size.to_bytes(1, 'big')


class _ANSIX923Unpadder(UnpadderContext):
    @staticmethod
    def _check(buffer, padded_size):
        for i in range(2, padded_size + 1):
            if buffer[-i] != 0:
                raise ValueError('invalid padding')


class ANSIX923(Padding):
    _padder_cls = _ANSIX923Padder
    _unpadder_cls = _ANSIX923Unpadder


# =================
# ISO 10126 Padding
# =================


class _ISO10126Padder(PadderContext):
    @staticmethod
    def _pad(padded_size):
        return os.urandom(padded_size - 1) + padded_size.to_bytes(1, 'big')


class _ISO10126Unpadder(UnpadderContext):
    @staticmethod
    def _check(buffer, padded_size):
        pass    # no need to check


class ISO10126(Padding):
    _padder_cls = _ISO10126Padder
    _unpadder_cls = _ISO10126Unpadder


# ==============
# Lookup Padding
# ==============


_PADDING_TABLE = {
    'PKCS7': PKCS7,
    'ANSIX923': ANSIX923,
    'ISO10126': ISO10126
}


def _lookup_padding(padding):
    if inspect.isclass(padding) and issubclass(padding, Padding):
        return padding
    if isinstance(padding, str):
        try:
            return _PADDING_TABLE[padding.upper()]
        except KeyError:
            pass
    raise ValueError('padding must be in {}'.format(set(_PADDING_TABLE)))



import inspect

from xycrypto.padding import _lookup_padding

from . import _lib

_MODE_TABLE = {
    'ECB': _lib.ECB,
    'CBC': _lib.CBC,
    'CFB': _lib.CFB,
    'OFB': _lib.OFB,
    'CTR': _lib.CTR
}


def _lookup_mode(mode):
    if inspect.isclass(mode) and issubclass(mode, _lib.Mode):
        return mode
    if isinstance(mode, str):
        try:
            return _MODE_TABLE[mode.upper()]
        except KeyError:
            pass
    raise ValueError('mode must be in {}'.format(set(_MODE_TABLE)))


def _setup_mode_padding(mode, **kwargs):
    mode = _lookup_mode(mode)

    args = {}
    if issubclass(mode, _lib.ModeWithInitializationVector):
        try:
            args['initialization_vector'] = kwargs['iv']
        except KeyError:
            raise TypeError('missing required keyword-only argument: "iv"')
    if issubclass(mode, _lib.ModeWithNonce):
        try:
            args['nonce'] = kwargs['nonce']
        except KeyError:
            raise TypeError('missing required keyword-only argument: "nonce"')

    # For ECB and CBC modes, the default padding is PKCS7.
    # For other modes, padding will not be added automatically.
    # However, user can force padding by providing the padding argument.
    if mode.name in {'ECB', 'CBC'}:
        padding = kwargs.pop('padding', 'PKCS7')
    else:
        padding = kwargs.pop('padding', None)

    return mode(**args), padding


def _determine_padding(padding, block_size):
    if padding is None:
        return None
    padding = _lookup_padding(padding)
    return padding(block_size)


def _determine_encryptor(cipher, padding):
    encryptor = cipher.encryptor()
    if padding is None:
        return encryptor
    return padding.wrap_encryptor(encryptor)


def _determine_decryptor(cipher, padding):
    decryptor = cipher.decryptor()
    if padding is None:
        return decryptor
    return padding.wrap_decryptor(decryptor)


def _make_X(cipher_attrs):
    def _X(cipher_base):
        name = cipher_base.__name__ + '_' + cipher_attrs['name']
        return type(name, (cipher_base,), cipher_attrs)
    return _X



import abc
import functools
import hashlib
import os

__all__ = [
    'MD5', 'SHA1', 'SHA224', 'SHA256', 'SHA384', 'SHA512',
    'SHA3_224', 'SHA3_256', 'SHA3_384', 'SHA3_512', 'SHAKE128', 'SHAKE256',
    'BLAKE2b', 'BLAKE2s'
]

_CHUNK_SIZE = 0x100000


class Hash(metaclass=abc.ABCMeta):
    """Abstract base class for hash context."""

    @property
    @abc.abstractmethod
    def _cls(self):
        """The class of hash context."""

    # ==================
    # Context Interfaces
    # ==================

    def __init__(self):
        """Initialize the current context."""

        self._ctx = self._cls()

    def update(self, data):
        """Update the current context."""

        self._ctx.update(data)

    def finalize(self):
        """Finalize the current context and return the message digest as bytes."""

        return self._ctx.digest()   # note the interface of stdlib

    def copy(self):
        """Copy the current context."""

        return self._ctx.copy()

    # ===============
    # Fast Interfaces
    # ===============

    @classmethod
    def hash(cls, data, **kwargs):
        """Return hash of data from byte string or unicode string."""

        if isinstance(data, str):
            data = data.encode('utf-8')
        ctx = cls(**kwargs)
        ctx.update(data)
        return ctx.finalize()

    @classmethod
    def hash_iter(cls, iterable, **kwargs):
        """Return hash of data from iterable of bytes."""

        ctx = cls(**kwargs)
        for chunk in iterable:
            ctx.update(chunk)
        return ctx.finalize()

    @classmethod
    def hash_fileobj(cls, fileobj, **kwargs):
        """Return hash of data from file object."""

        it = iter(functools.partial(fileobj.read, _CHUNK_SIZE), b'')
        return cls.hash_iter(it, **kwargs)

    @classmethod
    def hash_file(cls, filepath, **kwargs):
        """Return hash of data from file."""

        with open(filepath, 'rb') as f:
            return cls.hash_fileobj(f, **kwargs)

    @classmethod
    def hash_dir(cls, dirpath, **kwargs):
        """Return hash of data from directory."""

        try:
            digest_size = getattr(cls, 'digest_size')
        except AttributeError:
            digest_size = getattr(cls(**kwargs), 'digest_size')

        def _hash_dir(cls, dirpath, **kwargs):
            result = b'\x00' * digest_size
            with os.scandir(dirpath) as it:
                for entry in it:
                    if entry.is_dir():
                        value = cls.hash_dir(entry, **kwargs)
                    else:
                        value = cls.hash_file(entry, **kwargs)
                    result = bytes(x ^ y for x, y in zip(result, value))
            return result

        return _hash_dir(cls, dirpath, **kwargs)

    @classmethod
    def hash_fs(cls, path, **kwargs):
        """Return hash of data from filesystems."""

        if os.path.isdir(path):
            return cls.hash_dir(path, **kwargs)
        return cls.hash_file(path, **kwargs)


class ExtendableHash(Hash):
    """Abstract base class for extendable hash context."""

    @abc.abstractmethod
    def __init__(self):
        """Initialize the current context."""


class MD5(Hash):
    _cls = hashlib.md5
    block_size = 64
    digest_size = 16


class SHA1(Hash):
    _cls = hashlib.sha1
    block_size = 64
    digest_size = 20


class SHA224(Hash):
    _cls = hashlib.sha224
    block_size = 64
    digest_size = 28


class SHA256(Hash):
    _cls = hashlib.sha256
    block_size = 64
    digest_size = 32


class SHA384(Hash):
    _cls = hashlib.sha384
    block_size = 128
    digest_size = 48


class SHA512(Hash):
    _cls = hashlib.sha512
    block_size = 128
    digest_size = 64


class SHA3_224(Hash):
    _cls = hashlib.sha3_224
    block_size = 144
    digest_size = 28


class SHA3_256(Hash):
    _cls = hashlib.sha3_256
    block_size = 136
    digest_size = 32


class SHA3_384(Hash):
    _cls = hashlib.sha3_384
    block_size = 104
    digest_size = 48


class SHA3_512(Hash):
    _cls = hashlib.sha3_512
    block_size = 72
    digest_size = 64


class SHAKE128(ExtendableHash):
    _cls = hashlib.shake_128
    block_size = 168

    def __init__(self, *, digest_size=16):
        self._ctx = self._cls()
        self.digest_size = digest_size

    def finalize(self):
        return self._ctx.digest(self.digest_size)


class SHAKE256(ExtendableHash):
    _cls = hashlib.shake_256
    block_size = 136

    def __init__(self, *, digest_size=32):
        self._ctx = self._cls()
        self.digest_size = digest_size

    def finalize(self):
        return self._ctx.digest(self.digest_size)


class BLAKE2b(ExtendableHash):
    _cls = hashlib.blake2b
    block_size = 128
    max_digest_size = 64
    max_key_size = 64
    salt_size = 16
    person_size = 16

    def __init__(self, *, digest_size=64, key=b'', salt=b'', person=b'',
                 fanout=1, depth=1, leaf_size=0,
                 node_offset=0, node_depth=0,
                 inner_size=0, last_node=False):
        self._ctx = self._cls(
            digest_size=digest_size, key=key, salt=salt, person=person,
            fanout=fanout, depth=depth, leaf_size=leaf_size,
            node_offset=node_offset, node_depth=node_depth,
            inner_size=inner_size, last_node=last_node
        )
        self.digest_size = digest_size


class BLAKE2s(ExtendableHash):
    _cls = hashlib.blake2s
    block_size = 64
    max_digest_size = 32
    max_key_size = 32
    salt_size = 8
    person_size = 8

    def __init__(self, *, digest_size=32, key=b'', salt=b'', person=b'',
                 fanout=1, depth=1, leaf_size=0,
                 node_offset=0, node_depth=0,
                 inner_size=0, last_node=False):
        self._ctx = self._cls(
            digest_size=digest_size, key=key, salt=salt, person=person,
            fanout=fanout, depth=depth, leaf_size=leaf_size,
            node_offset=node_offset, node_depth=node_depth,
            inner_size=inner_size, last_node=last_node
        )
        self.digest_size = digest_size



from . import _base, _lib, _utils

_AES_ATTRS = {
    '_algorithm': _lib.AES,
    'name': 'AES',
    'block_size': 16,
    'key_sizes': frozenset([16, 24, 32])
}

_X = _utils._make_X(_AES_ATTRS)


class AES(_X(_base.BlockCipher)):
    pass


class AES_ECB(_X(_base.BlockCipherECB)):
    pass


class AES_CBC(_X(_base.BlockCipherCBC)):
    pass


class AES_CFB(_X(_base.BlockCipherCFB)):
    pass


class AES_OFB(_X(_base.BlockCipherOFB)):
    pass


class AES_CTR(_X(_base.BlockCipherCTR)):
    pass



import functools
import os
from hmac import compare_digest

__all__ = ['HMAC', 'compare_digest']

_CHUNK_SIZE = 0x100000
_TRANS_36 = bytes((x ^ 0x36) for x in range(256))
_TRANS_5C = bytes((x ^ 0x5C) for x in range(256))


class HMAC(object):
    """Hash-based Message Authentication Code."""

    # ==================
    # Context Interfaces
    # ==================

    def __init__(self, hash_cls, key):
        """Initialize the current context."""

        self._i_ctx = hash_cls()
        self._o_ctx = hash_cls()
        self.block_size = self._o_ctx.block_size
        self.digest_size = self._o_ctx.digest_size

        if len(key) > self.block_size:
            ctx = hash_cls()
            ctx.update(key)
            key = ctx.finalize()
        key = key.ljust(self.block_size, b'\0')

        self._i_ctx.update(key.translate(_TRANS_36))
        self._o_ctx.update(key.translate(_TRANS_5C))

    def update(self, data):
        """Update the current context."""

        self._i_ctx.update(data)

    def finalize(self):
        """Finalize the current context and return the message digest as bytes."""

        ctx = self._o_ctx.copy()
        ctx.update(self._i_ctx.finalize())
        return ctx.finalize()

    def verify(self, signature):
        """Finalize the current context and securely compare digest to signature."""

        return compare_digest(self.finalize(), signature)

    def copy(self):
        """Copy the current context."""

        other = type(self).__new__(type(self))
        other._i_ctx = self._i_ctx.copy()
        other._o_ctx = self._o_ctx.copy()
        other.block_size = self.block_size
        other.digest_size = self.digest_size
        return other

    # ===============
    # Fast Interfaces
    # ===============

    @classmethod
    def hash(cls, hash_cls, key, data, **kwargs):
        """Return hash of data from byte string or unicode string."""

        if isinstance(data, str):
            data = data.encode('utf-8')
        ctx = cls(hash_cls, key, **kwargs)
        ctx.update(data)
        return ctx.finalize()

    @classmethod
    def hash_iter(cls, hash_cls, key, iterable, **kwargs):
        """Return hash of data from iterable of bytes."""

        ctx = cls(hash_cls, key, **kwargs)
        for chunk in iterable:
            ctx.update(chunk)
        return ctx.finalize()

    @classmethod
    def hash_fileobj(cls, hash_cls, key, fileobj, **kwargs):
        """Return hash of data from file object."""

        it = iter(functools.partial(fileobj.read, _CHUNK_SIZE), b'')
        return cls.hash_iter(hash_cls, key, it, **kwargs)

    @classmethod
    def hash_file(cls, hash_cls, key, filepath, **kwargs):
        """Return hash of data from file."""

        with open(filepath, 'rb') as f:
            return cls.hash_fileobj(hash_cls, key, f, **kwargs)

    @classmethod
    def hash_dir(cls, hash_cls, key, dirpath, **kwargs):
        """Return hash of data from directory."""

        digest_size = getattr(cls(hash_cls, key, **kwargs), 'digest_size')

        def _hash_dir(cls, hash_cls, key, dirpath, **kwargs):
            result = b'\x00' * digest_size
            with os.scandir(dirpath) as it:
                for entry in it:
                    if entry.is_dir():
                        value = cls.hash_dir(hash_cls, key, entry, **kwargs)
                    else:
                        value = cls.hash_file(hash_cls, key, entry, **kwargs)
                    result = bytes(x ^ y for x, y in zip(result, value))
            return result

        return _hash_dir(cls, hash_cls, key, dirpath, **kwargs)

    @classmethod
    def hash_fs(cls, hash_cls, key, path, **kwargs):
        """Return hash of data from filesystems."""

        if os.path.isdir(path):
            return cls.hash_dir(hash_cls, key, path, **kwargs)
        return cls.hash_file(hash_cls, key, path, **kwargs)



from pathlib import Path

from setuptools import find_packages, setup

from xycrypto import __version__, __author__

readme = Path(__file__).with_name('README.md').read_text()

classifiers = [
    'Operating System :: OS Independent',
    'Programming Language :: Python',
    'Programming Language :: Python :: 3',
    'Programming Language :: Python :: 3.6',
    'Programming Language :: Python :: 3.7',
    'Programming Language :: Python :: 3.8',
    'Topic :: Security',
    'Topic :: Security :: Cryptography'
]

setup(
    name='xycrypto',
    version=__version__,
    license='MIT',

    description='A friendly cryptographic primitives library.',
    long_description=readme,
    long_description_content_type='text/markdown',

    author=__author__,
    author_email='thyfan@163.com',
    url='https://github.com/xymy/xycrypto',

    classifiers=classifiers,

    package_dir={'': 'src'},
    packages=find_packages('src'),
    include_package_data=True,

    install_requires=[
        'cryptography>=2.8',
    ],
    python_requires='>=3.6'
)



import abc

from . import _lib, _utils


class Cipher(metaclass=abc.ABCMeta):
    """Abstract base class for cipher."""

    @property
    @abc.abstractmethod
    def _algorithm(self):
        """The algorithm of cipher."""

    @abc.abstractmethod
    def __init__(self, algorithm, mode):
        """Prepare the cipher context."""

        self._cipher = _lib.Cipher(algorithm, mode, _lib.backend)

    @abc.abstractmethod
    def encryptor(self):
        """Return the encryptor context."""

    @abc.abstractmethod
    def decryptor(self):
        """Return the decryptor context."""

    def encrypt(self, data):
        """Encrypt data and return."""

        encryptor = self.encryptor()
        temp = encryptor.update(data)
        return temp + encryptor.finalize()

    def decrypt(self, data):
        """Decrypt data and return."""

        decryptor = self.decryptor()
        temp = decryptor.update(data)
        return temp + decryptor.finalize()

    @property
    def key_size(self):
        return self._cipher.algorithm.key_size // 8


class StreamCipher(Cipher):
    """Abstract base class for stream cipher."""

    def __init__(self, key):
        self._cipher = _lib.Cipher(self._algorithm(key), None, _lib.backend)

    def encryptor(self):
        return self._cipher.encryptor()

    def decryptor(self):
        return self._cipher.decryptor()


class BlockCipher(Cipher):
    """Abstract base class for block cipher."""

    @property
    @abc.abstractmethod
    def block_size(self):
        """The block size in bytes of cipher."""

    def __init__(self, key, mode, **kwargs):
        mode, padding = _utils._setup_mode_padding(mode, **kwargs)
        self._cipher = _lib.Cipher(self._algorithm(key), mode, _lib.backend)
        self._padding = _utils._determine_padding(padding, self.block_size)

    def encryptor(self):
        return _utils._determine_encryptor(self._cipher, self._padding)

    def decryptor(self):
        return _utils._determine_decryptor(self._cipher, self._padding)


class BlockCipherECB(BlockCipher):
    """Abstract base class for block cipher in ECB mode."""

    def __init__(self, key, *, padding='PKCS7'):
        self._cipher = _lib.Cipher(self._algorithm(key), _lib.ECB(), _lib.backend)
        self._padding = _utils._determine_padding(padding, self.block_size)


class BlockCipherCBC(BlockCipher):
    """Abstract base class for block cipher in CBC mode."""

    def __init__(self, key, *, iv, padding='PKCS7'):
        self._cipher = _lib.Cipher(self._algorithm(key), _lib.CBC(iv), _lib.backend)
        self._padding = _utils._determine_padding(padding, self.block_size)


@StreamCipher.register
class BlockCipherCFB(BlockCipher):
    """Abstract base class for block cipher in CFB mode."""

    def __init__(self, key, *, iv, padding=None):
        self._cipher = _lib.Cipher(self._algorithm(key), _lib.CFB(iv), _lib.backend)
        self._padding = _utils._determine_padding(padding, self.block_size)


@StreamCipher.register
class BlockCipherOFB(BlockCipher):
    """Abstract base class for block cipher in OFB mode."""

    def __init__(self, key, *, iv, padding=None):
        self._cipher = _lib.Cipher(self._algorithm(key), _lib.OFB(iv), _lib.backend)
        self._padding = _utils._determine_padding(padding, self.block_size)


@StreamCipher.register
class BlockCipherCTR(BlockCipher):
    """Abstract base class for block cipher in CTR mode."""

    def __init__(self, key, *, nonce, padding=None):
        self._cipher = _lib.Cipher(self._algorithm(key), _lib.CTR(nonce), _lib.backend)
        self._padding = _utils._determine_padding(padding, self.block_size)



from . import _base, _lib, _utils

_TripleDES_ATTRS = {
    '_algorithm': _lib.TripleDES,
    'name': '3DES',
    'block_size': 8,
    'key_sizes': frozenset([8, 16, 24])
}

_X = _utils._make_X(_TripleDES_ATTRS)


class TripleDES(_X(_base.BlockCipher)):
    pass


class TripleDES_ECB(_X(_base.BlockCipherECB)):
    pass


class TripleDES_CBC(_X(_base.BlockCipherCBC)):
    pass


class TripleDES_CFB(_X(_base.BlockCipherCFB)):
    pass


class TripleDES_OFB(_X(_base.BlockCipherOFB)):
    pass


class TripleDES_CTR(_X(_base.BlockCipherCTR)):
    pass
